package be.iminds.ilabt.jfed.lowlevel.api.test;

import be.iminds.ilabt.jfed.testing.base.ApiTestConfig;
import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import io.dropwizard.jackson.Jackson;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.util.Map;
import java.util.Objects;

@JsonInclude(JsonInclude.Include.NON_DEFAULT)
@JsonIgnoreProperties(value={"@context"})
public class GPULabTestConfig implements ApiTestConfig {
    private static final Logger LOG = LoggerFactory.getLogger(GPULabTestConfig.class);

    @Nonnull private final String gpulabURL;
    @Nonnull private final String version;

    private static final ObjectMapper MAPPER = Jackson.newObjectMapper();

    public GPULabTestConfig() {
        this.gpulabURL = "https://dev.gpulab.ilabt.imec.be/";
        this.version = "dev";
    }

    @JsonCreator
    public GPULabTestConfig(@JsonProperty("gpulabURL") @Nullable String gpulabURL,
                            @JsonProperty("version") @Nullable String version) {
        this.gpulabURL = gpulabURL == null ? new GPULabTestConfig().getGpulabURL() : gpulabURL;
        this.version = version == null ? new GPULabTestConfig().getVersion() : version;
    }



//    @JsonCreator
//    public static GPULabTestConfig createFromJsonObject(@Nonnull Map<String,Object> props) {
//        GPULabTestConfig res = MAPPER.convertValue(props, GPULabTestConfig.class);
//        return res;
//    }

    @Override
    public boolean isValid() {
        return gpulabURL.startsWith("https://");
    }

    @Nonnull
    @JsonProperty
    public String getGpulabURL() {
        return gpulabURL;
    }

    @Nonnull
    @JsonProperty
    public String getVersion() {
        return version;
    }

    @Nonnull
    @Override
    public String stringValue() {
        try {
            return MAPPER.writeValueAsString(this);
        } catch (JsonProcessingException e) {
            throw new RuntimeException("Failed to covert "+getClass().getName()+" to String", e);
        }
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        GPULabTestConfig that = (GPULabTestConfig) o;
        return Objects.equals(gpulabURL, that.gpulabURL) &&
                Objects.equals(version, that.version);
    }

    @Override
    public int hashCode() {
        return Objects.hash(gpulabURL, version);
    }
}
